/*
 *  This file is part of uIRC. (https://git.redxen.eu/caskd/uIRC)
 *  Copyright (c) 2019, 2020 Alex-David Denes
 *
 *  uIRC is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  any later version.
 *
 *  uIRC is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with uIRC.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "../public/assemblers.h"
#include "../public/commands.h"
#include "../public/types.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

int
main(void)
{
	char        mesg[513] = { 0 };
	IRC_Message input     = {
#ifdef UIRC_IRCV3
		.tags = { .msgid = { .value = "10" } },
#endif
		.name     = { .nick = "dad", .user = "dad-door", .host = "home.localhost" },
		.cmd      = PRIVMSG,
		.args     = { "(You)", "are ya winning son?", NULL },
		.trailing = true,
	};
	signed long res;
	if ((res = Assm_mesg(mesg, &input, 512)) <= 0) {
		printf("Failed to assemble message. (%li)\n", res);
		return EXIT_FAILURE;
	}
	char* expect =
#ifdef UIRC_IRCV3
	    "@msgid=10 "
#endif
	    ":dad!dad-door@home.localhost PRIVMSG (You) :are ya winning son?\r\n";
	if (strcmp(expect, mesg) != 0) {
		printf("Assembled message mismatch.\nGot:\n%s\nbut expected:\n%s\n", mesg, expect);
		return EXIT_FAILURE;
	}
	return EXIT_SUCCESS;
}

